pub struct PoolConfig {
pub min_connections: usize,
pub max_connections: usize,
pub idle_timeout_ms: u64,
pub acquire_timeout_ms: u64,
pub max_lifetime_ms: u64,
pub test_on_checkout: bool,
pub test_on_return: bool,
}Expand description
Connection pool configuration.
Fields
min_connections: usizeMinimum number of connections to maintain
max_connections: usizeMaximum number of connections allowed
idle_timeout_ms: u64Connection idle timeout in milliseconds
acquire_timeout_ms: u64Maximum time to wait for a connection in milliseconds
max_lifetime_ms: u64Maximum lifetime of a connection in milliseconds
test_on_checkout: boolTest connections before giving them out
test_on_return: boolTest connections when returning them to the pool
Implementations
Sourceimpl PoolConfig
impl PoolConfig
Sourcepub fn new(max_connections: usize) -> PoolConfig
pub fn new(max_connections: usize) -> PoolConfig
Create a new pool configuration with the given max connections.
Sourcepub fn min_connections(self, n: usize) -> PoolConfig
pub fn min_connections(self, n: usize) -> PoolConfig
Set minimum connections.
Sourcepub fn idle_timeout(self, ms: u64) -> PoolConfig
pub fn idle_timeout(self, ms: u64) -> PoolConfig
Set idle timeout in milliseconds.
Sourcepub fn acquire_timeout(self, ms: u64) -> PoolConfig
pub fn acquire_timeout(self, ms: u64) -> PoolConfig
Set acquire timeout in milliseconds.
Sourcepub fn max_lifetime(self, ms: u64) -> PoolConfig
pub fn max_lifetime(self, ms: u64) -> PoolConfig
Set max lifetime in milliseconds.
Sourcepub fn test_on_checkout(self, enabled: bool) -> PoolConfig
pub fn test_on_checkout(self, enabled: bool) -> PoolConfig
Enable/disable test on checkout.
Sourcepub fn test_on_return(self, enabled: bool) -> PoolConfig
pub fn test_on_return(self, enabled: bool) -> PoolConfig
Enable/disable test on return.